function s = chinremdds(f,n,z)
% s = newdds(f,n,z)
% Direkte Digitale Cos/Sin-Synthese durch komplexe Multiplikation
% mehrerer analytischer Cos/Sin-Signale aus unterschiedlichen von
% gegenseitig teilerfremden Tabellen.
%
%  f  :  Frequenz relativ zur Abtastfrequenz
%        Das Signal s wird analytisch berechnet.  
%        Deshalb ist  0 <= f < 1.0  zulssig.
%        (Default: Zufallszahl)
%        f kann ein abgetastetes Signal sein.  Dann wird die 
%        jeweilige Momentanfrequenz verwendet.
%  n  :  Anzahl zu berechnender Abtastschritte.  Ist f als Vektor 
%        gegeben, so wird anstelle von n die Lnge von f genommen.
%        In diesem Fall ist fr n ein beliebiger Wert einzusetzen.
%  z  :  Vektor der Lngen der Sinustabellen.
%        Die Komponenten von z mssen teilerfremd sein.
%        (Default [101 103 107 109])
%        Im Grenzfall ist z nur eine Zahl, die die Lnge einer Tabelle angibt.
%        Fr z = 0 wird der Sinus direkt in MATLAB gerechnet.
%
%  s  :  komplexes Ausgabesignal

% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1      f = 0.4*rand;      end
if length(f)>1   n = length(f);     elseif nargin<2  n = 1024; end
if length(f)<2   f = f*ones(1,n);   end
if nargin<3  z = [101 103 107 109]; end

% Direkte  Berechnung bei  z = 0
% ==============================
if z==0
   s = exp(i*2*pi*cumsum(f));
   break
end

% Konstanten
% ==========
lz = length(z);                    % Anzahl der Sinustabellen
df = 1/prod(z);                    % kleinster Frequenzschritt
nf = round(f./df);                 % f in Frequenzschritten

% CosSin-Tabellen
% ===============
for l = 1:lz
   exptab{l} = exp(2*pi*i*(0:z(l)-1)/z(l));
end

% Berechnung der Phasenschritte
% =============================
x  = diophant(z);                  % Lsung der diophantischen Gleichung
for l=1:lz
   pd(l,:) = mod(nf*x(l),z(l));    % Phasenschritte fr alle Tabellen
end

% Exp-Berechnung
% ==============
for l = 1:lz                       % individuelle DDS
   si(l,:) = exptab{l}(mod(cumsum(pd(l,:)),z(l))+1);
end
s  = prod(si,1);                   % Produkt aller
