function [s,y] = costasloop(f0,x,fg)
% [s,y] = costasloop(f0,x,fg)
% Costas-Loop mit folgenden Parametern:
%  f0    :  erwartete Trgerfrequenz relativ zur Abtastfrequenz (Default 0.2)
%  x     :  reelles Inputsignal (Default 200 zufllige Bits in BPSK auf f0)
%  fg    :  Grenzfrequenz des Regler-Tiefpasses relativ zur Nyquistfrequenz 
%           (Default 0.01*f0)
% Ausgegeben wird das demodulierte Signal s und die komplexe Oszillatorschwingung y.
% Wenn das Resultat nicht zugewiesen wird, gibt es zwei Graphiken sowie 
% die Audioausgabe von Input und rekonstruiertem Trgersignal
%
% Hinweis: diese Funktion wurde nur mit den Defaultwerten getestet.

% Autor:  Klaus von der Heide;  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  f0 = 0.2;  end
if nargin<2    
   t = 1:4000;
   symb = bitstuffing(0,2,randb(200,[-1 +1]),[-1 +1]);
   symb = symb(1:200);
   x = nfach(symb,20).*cos(2*pi*1.001*f0*t+2*pi*rand);  % + 0.1*randn(size(t));
end
if nargin<3  fg = 0.01*f0; end

% Filterberechnung
% ================
g1 = 20;
b1 = fir1(g1,0.5*f0);
[b2,a2] = ellip(2,0.2,60,fg); % Reglerfilter
a1 = a2(1);
a2 = a2(2:end);                 

% Regelung
% ========
gp  = 0.01;                      % proportionale Regelverstrkung
gi  = 0.00001;                   % integrale Regelverstrkung
y(1)= 1;                         % Anfangswert
phi = 0;                         % Anfangsphase
dp  = 2*pi*f0;                   % Phasenschritt
sr1 = zeros(length(b1),1);       % Zustandsregister FIR-Tiefpass
sr2 = zeros(length(b2),1);       % Zustandsregister Reglerfilter (FIR)
sr3 = zeros(length(a2),1);       % Zustandsregister Reglerfilter (IIR)
for t=1:length(x)
   sr1    = [x(t)*y(t); sr1(1:end-1)];       % Zustand FIR-Tiefpass
   u(t)   = b1*sr1;                          % FIR-Tiefpass
   s(t)   = real(u(t));                      % demoduliertes Signal
   sr2    = [s(t)*imag(u(t)); sr2(1:end-1)]; % Zustand Reglerfilter (FIR)
   r(t)   = b2*sr2 - a2*sr3;                 % Regelspannung
   dp     = dp - gi*r(t);                    % integrale Phasenregelung
   phi    = mod(phi + dp - gp*r(t),2*pi);    % Phasenakkumulation
   y(t+1) = exp(i*phi);                      % VCO-Schwingung
   sr3    = [r(t); sr3(1:end-1)];            % Zustand Reglerfilter (IIR)
end

y = y(2:end);

if nargout>0 break; end

sound(real(x),8000); pause(1)
sound(real(y),8000)

h3 = fullscreen('Einschwingen','w',3,[2 1; 1 1]);
axes('Position',[0 0.1 1 0.8])
plot(1:length(x),[10*r;s])
line([1 length(x)],[0 0],'Color',0.7*[1 1 1],'LineStyle',':')
axis([1 length(x) -2 +2]); axis off
set(gcf,'keypressfcn','xzoomvis');
title('Einschwingvorgang','FontSize',13) 
legend('Regelspannung','Demodulation')

mm = 200;
nn = length(x) - mm;
x  = x(nn:end);
y  = y(nn:end);
s  = s(nn:end);
r  = r(nn:end);

%h0 = fullscreen('Filter');
%filterplot({b1;{b2;[a1 a2]}})
%legend('Tiefpassfilter','Reglerfilter')

%h1 = fullscreen('Komplex');
%d = x - y;
%plot(real(d),imag(d))
%line([-1 0; +1 0],[0 -1; 0 +1],'Color',0.7*[1 1 1])
%axis square; axis off

h2 = fullscreen('Zeitbereich','w',3,[2 1; 2 1]);
axes('Position',[0 0.1 1 0.8])
plot(1:length(x),[delay(x,g1/2);10*r;s])
line([g1/2 mm],[0 0],'Color',0.7*[1 1 1],'LineStyle',':')
axis([g1/2 mm -2 +2]); axis off
set(gcf,'keypressfcn','xzoomvis');
title('Costas-Loop','FontSize',13) 
legend('Input','Regelspannung','Demodulation')
