function [bestv,mdist,udist,zdist] = entfaltdemo(g,b,u,s,sigma)
% [bestv,mdist,udist,zdist] = entfaltdemo(g,b,u,s,sigma)
% Simulation einer Faltungskodierung mit Intersymbolinterferenz, 
% Dekodierung durch vollstndige Suche nach grsster Korrelation.
%
%   g   :  Generatormatrix des Faltungskodes (Nullen und Einsen)
%   b   :  Koeffizienten der Intersymbolinterferenz (reell),
%          ein einzelner (beliebiger) Wert bedeutet keine ISI 
%   u   :  zu kodierende Informationsbits (maximal 16)
%   s   :  Sendesymbole fr 0 und 1 z.B. [-1 +1]
% sigma :  Standardabweichung des Rauschens
%
% bestv :  dekodierte Information
% mdist :  Abstandsquadrat des Empfangswortes vom zugehrigen Kodewort
% udist :  Abstandsquadrat des Empfangswortes vom korrekten Kodewort
% zdist :  Abstandsquadrat des Empfangswortes vom zweitnchsten Kodewort
%
% Die Abstandsquadrate sind normiert auf den Abstand 1 zwischen den 
% beiden Niveaus. Im Fall  mdist<udist  liegt ein Dekodierfehler vor.
%
% Es werden drei Bilder gezeigt:
% 1. Das Abstandsquadrat der Kodewrter smtlicher Bitmuster 
%    derselben Lnge wie u vom simulierten Sendesignal
% 2. Die kumulative Verteilung der Abstandsquadrate
% 3. Die Verteilung der Abstandsquadrate ber dem Hammingabstand 
%    zwischen u und dem jeweiligen Bitmuster
%
% Beispiel: 
% entfaltdemo( faltpoly(2,5), [0.3 1 0.3], randb(10), [-1 +1], 0.3)

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<5  sigma = 1.0;           end
if nargin<4      s = [-1 +1];       end
if nargin<3      u = randb(10);     end
if nargin<2      b = 1.0;           end
if nargin<1      g = faltpoly(3,7); end
if length(b)<2   b = 1.0;           end
if length(u)>16  
   warning('Vektor u nach 16 Bits abgeschnitten!'); 
   u = u(1:16);
end

% Konstanten
% ==========
[cr,cl] = size(g);            % inverse Koderate cr und Constraint Length cl
bl  = length(b)-1;            %
n   = length(u);              % Lnge des Bitmusters
fsz = 13;                     % FontSize fr Darstellung

% Sendesignal
% ===========
x = s(faltcode(u,g)+1);       % Faltungskodierung
for r=1:cr
   y(r,:) = conv(b,x(r,:));   % Intersymbolinterferenz
end
z = y + sigma*randn(size(y)); % Rauschen 

% Berechnung des Abstandes fr alle Bitmuster
% ===========================================
for k=1:2^n
   nbits = num2binvec(k-1,n); % Bitmuster
   dist(k) = 0;
   for r=1:cr  
      symb(r,:) = s(1 + rem(conv(nbits,g(r,:)),2)); % Kodewort
      dist(k) = dist(k) + sum((z(r,:)-conv(b,symb(r,:))).^2);
   end
end
dist = dist/abs(s(1)-s(2))^2;  % Normierung auf Niveauabstand 1

% Ausgabe
% =======
mdist = min(dist);
udist = dist(binvec2num(u)+1);
bestk = min(find(dist==mdist))-1;
cabst = min(dist(find(dist>mdist))) - mdist;
zdist = cabst + mdist;
bestv = num2binvec(bestk,n);
muster= binmatout(bestv);

% Darstellung
% ===========
fh1 = fullscreen('Distanz');
plot(0:2^n-1,dist)
axis([0 2^n-1 0 max(dist)])
xlabel('Ganzzahlinterpretation des Bitmusters','FontSize',fsz)
ylabel('Abstandsquadrat','FontSize',fsz)
title(sprintf('Das Bitmuster  %s  (dezimal %5d)  liefert den kleinsten Abstand zum Input',...
      muster(1:end-1),bestk),'FontSize',fsz)
   
fh2 = fullscreen('kumulative Verteilung');
plot(sort(dist),(0:2^n-1)*2^-n)
axis([0 max(dist) 0 1])
xlabel('Abstandsquadrat','FontSize',fsz)
title('kumulative Verteilung der Abstandsquadrate','FontSize',fsz)
   
fh3 = fullscreen('Verteilung');
mnabs = inf;
for k=1:2^n-1
   hgabst = sum(u~=num2binvec(k-1,n));
   line(hgabst,dist(k),'Marker','*','LineStyle','none')
end
line([0 n],dist(1+binvec2num(u))*[1 1],'Color','r','LineStyle',':')
axis([0 n 0 max(dist)])
xlabel('Hammingabstand der Informationswrter','FontSize',fsz)
ylabel('Abstandsquadrat der Kodewrter','FontSize',fsz)
title('Verteilung der Abstandsquadrate der Kodewrter ber dem Hammingabstand der Informationsmuster','FontSize',fsz)

korrekt   = binmatout(u)
dekodiert = muster
Kodeabstand = cabst
