function  br = faltfrate(g,ebn0db,n)
% br = faltfrate(g,ebn0db,n)
% Ermittlung der Bitfehlerrate unter Einsatz eines Faltungskodes 
%
%     g  :  Matrix der Faltungspolynome,
%           anstelle von g kann auch ein Vektor mit den zwei 
%           Elementen [r, cl] eingegeben werden:
%           r  :  inverse Koderate,  cl  :  constraint length
%           In diesem Fall wird der Kode mit faltpoly erzeugt.
%
% ebn0db :  Eb/No in dB
%
%     n  :  Anzahl der Informationsbits in der Simulation
%
%    br  :  Bitfehlerrate bezogen auf die bertragenen Informationsbits
% 
% siehe auch faltfrate.m

% Autor: Klaus von der Heide, e-mail:  v.d.heideqon-line.de
%
%
% Defaults
% ========
if nargin<3       n = 10000; end
if nargin<2  ebn0db = 3;     end
if nargin<1       g = [2,5]; end

% Konstanten
% ==========
s = [-1 +1];
[cr,cl] = size(g);               % inverse Koderate und Constraint Length
if cr<2                          % Fall Eingabe von r und cl
   cr = g(1);
   cl = g(2);
   g  = faltpoly(cr,cl);         % Faltungspolynome
end
fsz = 13;                        % FontSize
del = 0.5;                       % Auflsung der Fehlerverteilung
nn = ceil((16000-cr*cl)/cr);     % Anzahl der Informationsbits pro viterbi
kk = ceil(n/nn);                 % Anzahl der Aufrufe viterbi
sigma = ebn2sigma(1/cr,ebn0db);  % Standardabweichung des Rauschens 
zhn = 2^cl;                      % Anzahl der Zustnde
zhnh= zhn/2;                     % davon die Hlfte
for k=1:zhn                      % das letzte Bit der Kodierung aller Zustandsmuster
   y = 2*faltcode(reverse(num2binvec(k-1,cl)),g)-1;
   for r=1:cr  z(r,k) = y(r,cl); end;
end
k0 = 1:zhnh;
k1 = k0 + zhnh;
p1 = 2*k0;
p0 = p1 - 1;

% Trellisanfang
% =============
for k=1:zhn                      % alle Zustandsmuster als Anfangswege (reverse)
   uu    = reverse(num2binvec(k-1,cl));
   bw0(k)= binvec2num(uu);
   xx    = 2*faltcode(uu,g)-1;
   x0{k} = xx(:,1:cl);
end
zw0 = zeros(1,zhn);              % komprimierte Wegereprsentation
rvk0= ones(1,zhn);               % Rckverkettung

% Simulation
% ==========
t0  = clock;
nf = 0;
for k=1:kk 
   u = randb(nn);                % zufllige Information
   x = 2*faltcode(u,g)-1;        % Faltungskodierung
   y = x + sigma*randn(size(x)); % Kanalrauschen
   x = [y ones(cr,1)*s(1)];      % eine Null-Spalte an Tail anhngen
   [xnz xns] = size(x);

   % Trellisanfang
   bw  = bw0;
   zw1 = zw0;;                   % komprimierte Wegereprsentation
   zw2 = zw0;                    % komprimierte Wegereprsentation
   zw3 = zw0;                    % komprimierte Wegereprsentation
   zlr = cl;                     % Zhler fr Wegekompression
   zl2 = 0;                      % Zhler fr Wegekompression
   d   = [];
   rvk1= rvk0;                   % Rckverkettung
   rvk2= rvk0;                   % Rckverkettung
   rvk3= rvk0;                   % Rckverkettung
   for l=1:zhn                   % alle Zustandsmuster als Anfangswege (reverse)
      c(l) = sum(sum((x0{l} - x(:,1:cl)).^2)); 
   end

   % Trellisschritt
   for t=cl+1:xns                % Zeitschleife ber Kodelnge
      c0 = c(p0);                % Kosten des ersten Vorgngerzustands
      c1 = c(p1);                % Kosten des zweiten Vorgngerzustands
      xb = (ones(zhnh,1)*x(:,t)')';
      s0 = sum((xb - z(:,k0)).^2);  % Kosten einer 0 an t-ter Stelle
      s1 = sum((xb - z(:,k1)).^2);  % Kosten einer 1 an t-ter Stelle
      b0 = c0<c1;                % Vergleich der Kosten
      b1 = 1 - b0;
      pb = p0.*b0 + p1.*b1;
      cb = c0.*b0 + c1.*b1;
      c(k0)   = cb + s0;
      bwn(k0) = 2*bw(pb);
      rvkn(k0)= rvk1(pb);
      c(k1)   = cb + s1;
      bwn(k1) = bwn(k0) + 1;
      rvkn(k1)= rvk1(pb);
      bw      = bwn;
      rvk1    = rvkn;
      % Wegekompression
      zlr = zlr + 1;
      if rem(zlr,16)==0 
         if zlr==48
            rvk = rvk3(rvk2(rvk1(1)));
            if any(rvk~=rvk3(rvk2(rvk1(2:zhn)))) 
               warning('Weg nach 144 Schritten noch nicht entschieden.')
            end
            d = [d  num2binvec(zw3(rvk),48)];
            zw3 = zw2;
            zw2 = zw1;
            zw1 = bw';
            rvk3= rvk2;
            rvk2= rvk1;
            rvk1= 1:zhn;
            zlr = 0;
         end    
         bw  = rem(bw,2^32);        
      end
   end
   % Ausgabe
   d = [d  num2binvec(zw3(rvk3(rvk2(rvk1(1)))),48)   ...
           num2binvec(zw2(rvk2(rvk1(1))),48)  ...
           num2binvec(zw1(rvk1(1)),48)  ...
           num2binvec(rem(bw(1),2^zlr),zlr)];
   d = d(145:end-cl);
   
   nf  = nf + sum(d~=u);        % Anzahl der Bitfehler in der dekodierten Information
   
   if rem(k,100)==0             % Datenrettung
      anzahlinfobits  = k*nn
      anzahlbitfehler = nf
      bitfehlerrate   = nf/(k*nn)
      save 'frate.mat' k nn g ebn0db nf
   end
end

br = sum(nf)/kk/nn;             % BER
