function  [d,s] = fsk441rx(x,fs,df0,ddf)
%  [d,s] = fsk441rx(x,fs,df0,ddf) 
%  decodiert das FSK-Signal nach dem Standard FSK441 von 
%  Joe Taylor (Nobel-Preis fr Physik 1993) fr die Abtastrate 8820 Hz.
%  x   :  Vektor des zu decodierendes FSK-Signal
%         Anstelle von x und fs kann der Name einer Wave-Datei (mit oder ohne .WAV)
%         angegeben werden.
%  fs  :  Abtastfrequenz von x (ohne Bedeutung, falls x Wave-Datei ist).
%  df0 :  Frequenzfehler des Signals in Hz
%  ddf :  Frequenzdrift des Signals oder Chirp eines Pings in Hz/s
%
%  d   :  decodierter String
%  s   :  Signifikanz der Zeichen von d
%
%  Wird fsk441rx ohne Parameter aufgerufen, so wird ein 200 ms langer 
%  Ping mit 12 dB erzeugt mit dem Text "CQ K1JT ".
%
%  Wird d nicht auf eine Variable zugewiesen, so gibt es graphische 
%  Darstellungen zur Signalverarbeitung.
%
%  Falls x bzw. die Wave-Datei nicht mit fs = 8820 Hz abgetastet ist, 
%  wird auf 8820 Hz umgetastet.

%  Autor:   Klaus von der Heide, DJ5HG, e-mail:  v.d.heide@on-line.de

%  Defaults und Konstanten
%  =======================
if nargin<4  ddf = 0;    end
if nargin<3  df0 = -20;  end
if nargin<2   fs = 8820; end
if nargin<1    x = fsk441kanal(fsk441tx('cq K1JT',5000,8820),8820,12,0.2); end
if ischar(x)  [x,fs] = wavreadx(x); x = x'; end
fsi= 8820;                       % interne Abtastrate
fb = 441;                        % Baudrate
f1 = 441;                        % Grundfrequenz 
fsiz = 12;                       % FontSize
tc2  = 0.01;                     % Zeitkonstante des Pingsensors [s]
triggerlevel =  1.2;             % Ping-Trigger-Ebene (Amplitudenquadrat bezogen auf dessen Mittelwert)
begrenzung   =  8.0;             % Begrenzung (Amplitudenquadrat bezogen auf dessen Mittelwert)
grb = 100;                       % Filtergrad fr Bandpassfilter
grc = 60;                        % Filtergrad fr Schrittfrequenztiefpassfilter
stp = 20;                        % Symbolschritt in Abtastungen

% Abtastrate auf fsi bringen
% ==========================
if fs~=fsi
   if floor(fs)==fs & floor(fsi)==fsi
      kgv = lcm(fs,fsi);
      if kgv<200000
         fl = 2*5.5*f1/kgv;                            % Durchlassbereich des Interpolationsfilters
         fu = 2*min([fs fsi])/kgv - fl;                % Sperrbereich des Interpolationsfilters
         b1 = remez(62,[0 fl fu 1],[1 1 0 0],[1 25]);  % Interpolationsfilter
         xh = filter(b1*kgv/fs,1,nullein(x,kgv/fs-1)); % Hochtastung auf kgv
         xi = xh(1:kgv/fsi:end);                       % Tieftastung auf fsi
         resamp = 0;
      else 
         resamp = 1;
      end
   else 
      resamp = 1;
   end
   if resamp
      bu = fir1(62,5.5*f1/(fs/2));                    % Interpolationsfilter
      t  = 1:length(x);                               % Zeitbereich in Einheiten 1/fs
      xi = umtastung(x,t/fs,(1:floor(t(end)*fsi/fs))/fsi,bu,4);
   end
else
   xi = x;
end
t = 1:length(xi);                % Zeitbereich in Einheiten 1/fsi

% Frequenzfehler
% ==============
df  = df0 + ddf*t/fsi;
xi  = real2analyt(xi,10).*exp(-i*2*pi*(rand+df.*t/fsi));

% Decodiertabelle fr je drei Tne in Character
% =============================================
ctab = '{123456789.,?/# $ABCD[FGHIJKLMNOPQRSTUVWXY]0EZ-----------------}';

% Tonmatrix: 4 Tne  (k+1)*441 Hz  (k=1,2,3,4)
% ============================================
for k=1:4  tonmatrix(k,:) = sin(2*pi*(k+1)*((1:stp)-0.5)/stp);  end

% Bandpassfilterung auf 500 ... 2500 Hz
% =====================================
bb = remez(grb,[0 200 500 2500 2800 fsi/2]/(fsi/2),[0 0 1 1 0 0]);
y = filter(bb,1,xi);

% Pingdetektion und Amplitudenbegrenzung
% ======================================
yy = y.^2;                       % Amplitudenquadrat
[b2,a2] = butter(4,1/tc2/fsi);   % Glttungsfilter
gd = grpdelay(b2,a2,512);        % Gruppenlaufzeit
dl = round(max(gd));             % Filterlaufzeit in Abtastschritten
am = filter(b2,a2,abs(yy));      % geglttete Amplitude
av = mean(am);                   % mittlere Amplitude
ax = max(am);                    % maximale Amplitude
tr = triggerlevel*av;            % Trigger-Ebene
gr = begrenzung*av;              % Begrenzungs-Ebene
k1 = floor(100*tr/gr/2);
k2 = floor(100*tr/gr);
k3 = 100;
k4 = 10000;
ktab(1:k1)    = 0;               % Kompressionstabelle
ktab(k1:k2-1) = raisedcosine([0 1],k2-k1);
ktab(k2:k3)   = 1;
ktab(k3:k4)   = k3./(k3:k4);     % Kompressionstabelle
z  = delay(y,dl).*ktab(round(1+k3*am/gr)); % Kompression 

y = z;

% Korrelation
% ===========
for n=1:length(xi)-stp;
   corr  = abs(tonmatrix*y(n+(0:stp-1))');
   c1(n) = corr(1);
   c2(n) = corr(2);
   c3(n) = corr(3);
   c4(n) = corr(4);
end
bc = remez(grc,[0 fb 1.5*fb fsi/2]/(fsi/2),[1 1 0 0]);
c1 = filter(bc,1,c1);
c2 = filter(bc,1,c2);
c3 = filter(bc,1,c3);
c4 = filter(bc,1,c4);

% Schrittsynchronisation
% ======================
pc = c1.*c2.*c3.*c4;             % Produkt der Korrelationen enthlt die Schrittfrequenz
[s,fschritt,cas,sas] = fitsin(pc,fb/fsi,0.00001);
phi = angle(cas+i*sas);
p = round(1+(pi+phi)/2/pi/fschritt:1/fschritt:length(xi));
if p(1)<1  p = p(2:end); end
if p(end)>length(c1)  p = p(1:end-1); end

% Decodierung
% ===========
for n=1:length(p)
   j     = p(n);
   cn    = [c1(j),c2(j),c3(j),c4(j)];
   cm(n) = max(cn);
   ix(n) = min(find(cn==cm(n)));
end
ib = [];
for n=1:length(p)-2  
   if ix(n:n+2)==[0 3 3]+1       % Space finden
      ib = [ib n];               % alle Indices von Space 
   end; 
end

if isempty(ib)  d = []; break; end

for n=1:length(ib)  pcm(n) = prod(cm(ib(n)+(0:2))); end
ibm = find(pcm==max(pcm));
dly = -mod(ib(ibm)-1,3);
nc  = floor(length(p)/3);
irx = reshape(delay(ix(1:3*nc),dly),3,nc)'; 
irx(nc,3+dly:3) = 3;
txt = [];
for n=1:nc-2  txt = [txt ctab(1+[16 4 1]*(irx(n,:)'-1))]; end

% Signifikanz
% ===========
sigtab = [zeros(1,5), raisedcosine([0 1],10), ones(1,10000)];

for n=2:length(txt)  
   g(n) = sigtab(1+ceil(30*sqrt(sum(am(p(3*n-2)+(0:3*stp-1))))/triggerlevel));
end

% Ausgabe
% =======
d = txt(2:end);
g = g(2:end);

% Darstellung und Audioausgabe
% ============================
if nargout<1
   
   fh1 = fullscreen('Korrelation','w',2,[5 1; 1 1]);
   axes('Position',[0.03 0.15 0.94 0.7])
   plot([c1;c2;c3;c4]','LineWidth',2);
   line(p,cm,'LineStyle','none','Marker','.','Color','k','MarkerSize',15)
   xlabel('Abtastung','FontSize',fsiz)
   axis([1 length(c1) 0 1.2*max(cm)])
   legend('f1','f2','f3','f4','Schrittsync')
   title('Korrelation der 4 Tne mit dem Signal','FontSize',fsiz)
   
   fitsin(pc,fb/fsi,0.00001);
   figure(max(findobj('Type','figure','Name','Approximation')))
   setfigpos(5,1,2,1)
   set(gcf,'Name','Schrittsync')
   title('Schrittsynchronisation durch Sinus-Fit in das Produkt der 4 Korrelationen','FontSize',fsiz)
   legend('Produkt der 4 Korrelationen','Sinus-Approximation')
   
   figure(max(findobj('Type','figure','Name','Spektrum')))
   line(0.05*[1 1],get(gca,'YLim'),'LineStyle',':','Color','r')
   title('Spektrum des Produktes der 4 Korrelationen aus Bild "Korrelation"','FontSize',fsiz)
   set(gcf,'Name','Sync-Spektrum')
   set(gca,'XScale','lin')
   setfigpos(5,1,3,1)
   legend('Spektrum','erwartete Schrittfrequenz')
   
   ts = (0:length(y)-1)/fsi;
   fh4 = fullscreen('Signal','w',2,[5 1; 4 1]);
   axes('Position',[0.03 0.15 0.94 0.7])
   plot(ts,real(xi));
   line(ts',delay(real(z),-dl-grb/2)','Color','m','LineWidth',2.0)
   line(ts',delay(sqrt(am),-dl-grb/2)','Color','g','LineWidth',2.0)
   line([0 ts(end)],sqrt(tr)*[1 1],'Color','r','LineStyle',':')
   line([0 ts(end)],sqrt(gr)*[1 1],'Color','r','LineStyle',':')
   line([1;1]*(p-1-dl-grb/2-grc/2)/fsi,[-1;1]*ones(size(p)),'Color','k')
   line([0 ts(end)],[0 0],'Color','w')
   axis([0 ts(end) 1.1*max(abs(xi))*[-1 +1]])
   title('Signal im Zeitbereich','FontSize',fsiz)
   xlabel('Zeit [s]','FontSize',fsiz)
   legend('Signal','vorverarbeitetes Signal','geglttete Amplitude',...
          'Trigger-Niveau','Begrenzer-Niveau','Schritt-Grenzen')
   
   fh5 = fullscreen('Signalspektrum','w',0,[5 1; 5 1]);
   axes('Position',[0.03 0.1 0.94 0.8])
   ff = abs(fft(y.*hamming(length(y))'));
   nh = floor(length(ff)/2);
   plot((0:nh-1)*fsi/2/nh,20*log10(ff(1:nh)/max(ff)))
   axis([0 fsi/2 -60 10])
   title('Spektrum des Signals (dB)','FontSize',fsiz)
   xlabel('Frequenz [Hz]','FontSize',fsiz)
   
   fh6 = fullscreen('Decodierung','w',0,[20 1; 20 1]);
   axes('Position',[0.03 0.1 0.94 0.8]); 
   axis([0 length(d)+1 -1 +1]);  axis off
   for n=1:length(d)  
      text(n,0.5,d(n),'FontWeight','bold','Color',(1-g(n))*[1 1 1],...
                      'Interpreter','none','FontName','FixedWidth'); 
   end
   
   sound(x,fs); 
   if abs(df0)>220 | abs(ddf)>10  pause(2); sound(real(xi),fsi); end
end
