function  x = fsk441tx(s,xl,fs)
%  x = fsk441tx(s,xl,fs) 
%  generiert das FSK-Signal  x  nach dem Standard FSK441 von 
%  Joe Taylor (Nobel-Preis fr Physik 1993) fr die Abtastrate fs.
%  s  :   zu sendender String. Ein fehlendes Space an Anfang oder Ende
%         wird automatisch angefgt.  s wird periodisch wiederholt. 
%         s darf folgende Buchstaben, Ziffern und Zeichen enthalten:
%         ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,?/# $
%         Der String wird allerdings mit UPPER gewandelt, so dass 
%         auch Kleinschreibung mglich ist.
%         Mit den Zeichen {,[,],} werden die reservierten Tonsequenzen 
%         000, 111, 222, 333 erzeugt. Besteht s nur aus einem einzigen 
%         dieser vier Zeichen, so wird kein Space angefgt.
%         fsk441tx('{') liefert also den reinen Ton 882 Hz.
%  xl :   maximale Lnge des Ausgabevektors in Abtastschritten
%  fs :   Abtastrate in Hz. Zulssig sind nur Abtastraten  k*441,
%         z.B. 7056, 8820, 11025, 22050, 44100.  Andernfalls wird zu 
%         diesen Abtastraten hin gerundet.

% Autor:   Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

%  Defaults und Konstanten
%  =======================
if nargin<3 fs = 8820; end
if nargin<2 xl = 5000; end
if nargin<1  s = '73'; end
fb = 441;                              % Baudrate
f1 = 441;                              % Grundfrequenz
na = round(fs/f1);                     % Anzahl Abtastungen pro Schritt

%  Jeder Character wird umgesetzt in eine Folge von drei Tnen.
%  Die Tabelle ttab liefert zu jedem Character die Indices dieser Tne:
%  ====================================================================
ttab = 4*ones(256,3);                  % vorbesetzen mit 4 fr "undefined"
ttab(48:57,:)       = [2 2 3; 0 0 1; 0 0 2; 0 0 3; 0 1 0; 0 1 1; 0 1 2; 0 1 3; 0 2 0; 0 2 1];
ttab(double('A'),:) = [1 0 1];
ttab(double('B'),:) = [1 0 2];
ttab(double('C'),:) = [1 0 3];
ttab(double('D'),:) = [1 1 0];
ttab(double('E'),:) = [2 3 0];
ttab(double('F'),:) = [1 1 2];
ttab(double('G'),:) = [1 1 3];
ttab(double('H'),:) = [1 2 0];
ttab(double('I'),:) = [1 2 1];
ttab(double('J'),:) = [1 2 2];
ttab(double('K'),:) = [1 2 3];
ttab(double('L'),:) = [1 3 0];
ttab(double('M'),:) = [1 3 1];
ttab(double('N'),:) = [1 3 2];
ttab(double('O'),:) = [1 3 3];
ttab(double('P'),:) = [2 0 0];
ttab(double('Q'),:) = [2 0 1];
ttab(double('R'),:) = [2 0 2];
ttab(double('S'),:) = [2 0 3];
ttab(double('T'),:) = [2 1 0];
ttab(double('U'),:) = [2 1 1];
ttab(double('V'),:) = [2 1 2];
ttab(double('W'),:) = [2 1 3];
ttab(double('X'),:) = [2 2 0];
ttab(double('Y'),:) = [2 2 1];
ttab(double('Z'),:) = [2 3 1];
ttab(double('.'),:) = [0 2 2];
ttab(double(','),:) = [0 2 3];
ttab(double('?'),:) = [0 3 0];
ttab(double('/'),:) = [0 3 1];
ttab(double('#'),:) = [0 3 2];
ttab(double(' '),:) = [0 3 3];
ttab(double('$'),:) = [1 0 0];
ttab(double('{'),:) = [0 0 0];
ttab(double('['),:) = [1 1 1];
ttab(double(']'),:) = [2 2 2];
ttab(double('}'),:) = [3 3 3];

% Stringprfung
% =============
if ismember(4,ttab(double(upper(s))))  error('Der String enthlt nicht erlaubte Zeichen.'); end
if s(1)~=' ' & s(end)~=' ' & ~(ismember(s(1),'{[]}') & length(s)==1)
   s = [s ' '];                        % gegebenfalls Space anhngen
end

% Tonmatrix: 4 Tne  (k+1)*441 Hz  (k=1,2,3,4)
% ============================================
for k=1:4  tonmatrix(k,:) = sin(2*pi*(k+1)*((1:na)-0.5)/na);  end

% Sequenz des Tonindex
% ====================
sequenz = 1 + flach(ttab(double(upper(s)),:));

% FSK-Signal
% ==========
z = flach(tonmatrix(sequenz,:));

% Periodische Wiederholung
% ========================
x = flach(ones(floor(xl/length(z)),1)*z);
