function [xx,cr,x] = p3drx(y,f1,vis)
% [xx,cr,x] = p3drx(y,f1,vis)
% Telemetrie-Empfnger-Simulation des Satelliten P3-D (AMSAT OSCAR-40)
% Im Signal y wird ein Block gesucht und Text dekodiert.
%
%  y  :  PSK-Signal mit Trgerfrequenz 1.5 kHz
%        Default: ASCII-Text 'P3-D: Test' entsprechend moduliert
%        Es kann auch als String der Name einer *.WAV-Datei angegeben werden.
%
%  f1 :  Trgerfrequenz:
%        f1 = 0 oder nicht angegeben:  Default = 1500 + 3*randn + 5*randn*t
%        0 < f1 < 4000 :  Diese Trgerfrequenz wird im Empfnger angenommen
%                         Die Trgerfrequenzsynchronisation arbeitet dann 
%                         bis zu Abweichungen von 50 Hz korrekt.
%        f1 = [fu fo] :   Die Trgerfrequenz wird im Bereich fu...fo gesucht.
%
%  vis:  Binrvektor zur Steuerung der Visualisierung:
%        vis(1):  Animation Signalraum z nach dem Empfangsfilter
%        vis(2):  Animation Signalraum z^m
%        vis(3):  Animation Signalraum nach der Korrektur
%        vis(4):  Phasengang des Trgers und Phasengang des Schrittes
%        vis(5):  Soundausgabe des Empfangssignals vor dem Filter
%        vis(6):  Animation zur Schrittsynchronisation entsprechend zu 1
%        vis(7):  Animation zur Schrittsynchronisation entsprechend zu 2
%        vis(8):  Schritt im Zeitbereich 
%        vis(9):  Frequenzgang der Filter
%        vis(10): Signale im Zeitbereich
%        Default: [0 0 0 1 1 0 0 0 0 0]
%
%  xx :  Interpretation der empfangenen Bits als ASCII-Text
%
%  cr :  CRC (ist 0 bei korrektem Enpfang des gesamten Blocks)
%
%  x  :  empfangene Bits
%
% !!! Die Dekodierung ist noch nicht optimal.

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  vis = [0 0 0 1 1 0 0 0 0 0];  end    % Visualisierung
if nargin<1    y = p3dtx('P3-D: Test ',0.5); end  % Sendesignal
if ischar(y)   y = wavreadx(y)'; end

% Konstanten
% ==========
fs = 800;                        % Schrittfrequenz
ly = length(y);
m  = 2;                          % Anzahl der Stufen
fa = 8000;                       % Abtastfrequenz
t  = (0:ly-1)/fa;                % Zeitbereich
as = ly*fs/fa;                   % Anzahl der Symbole im Block
g  = 60;                         % Filtergrad fr Empfangsfilter
p  = 2*pi*rand;                  % beliebige Phase
b  = firrcos(g,1.6*fs/fa,0.1);   % Entwurf des Empfangsfilters
k  = 2*g:ly;
kk = g+200:ly;
kkk= kk(1:6:end);
pgt= 2 + round(ly/10000);        % Polynomgrad fr Trgersync
pgs= 1 + round(ly/50000);        % Polynomgrad fr Schrittsync

% Trgerfrequenz
% ==============
if nargin<2  f1 = 1500 + 3*randn + 5*randn*t; end
if f1<1      f1 = 1500 + 3*randn + 5*randn*t; end
if length(f1)==2  f1 = fa/m*fmaxkomp(y.^m,m*f1(1)/fa,m*f1(2)/fa); end

% Empfnger
% =========
yb = y.*exp(i*2*pi*f1.*t+p);     % Transformation ins Basisband
z  = filter(b,1,yb);             % Empfangsfilter

% Trgersynchronisation
zsq= z.^m;                       % m-te Potenz
[bt,at] = ellip(2,0.5,60,0.002);
tsq= filter(bt,at,zsq);          % Glttung von zsq (sonst geht unwrap nicht)
c  = polyfit(t(kkk),unwrap(angle(tsq(kkk))),pgt); % Polynom-Fit in Phasengang
zz = z(k-g/2).*exp(-i*(polyval(c,t(k)))/m); % Korrektur

% Schrittsynchronisation
zzsq = (zz.^2).*exp(-i*2*pi*fs*t(k));
[bs,as] = ellip(2,0.5,58,0.0008);% IIR-Filter fr zsq-Glttung
ztsq = filter(bs,as,zzsq);       % Glttung von zzsq
zc   = polyfit(t(kkk),unwrap(angle(ztsq(kkk-2*g+1))),pgs); % Polynom-Fit in Phasengang
schr = sin(2*pi*fs*t(k)+polyval(zc,t(k)));
sclk = [0 diff(schr>0)>0];

% Datenentscheidung und Dekodierung
x  = dediffencode(demanchester(real(zz(find(sclk>0))>0)));

% Block-Dekodierung
ix = suchmuster(x,[0 0 1 1 1 0 0 1 0 0 0 1 0 1 0 1 1 1 1 0 1 1 0 1 0 0 1 1 0 0 0 0],4);
if isnan(ix) 
   xx = ''; 
else 
   xx = binvec2str(x(ix+32:ix+31+8*min(512,floor((length(x)-ix-31)/8)-1)));
   st = xx;
   if length(x)>ix+4142
      cr = binvec2num(crc(x(ix+32:ix+4143),[16 12 5 0])) % CRC-Berechnung
   end
   str = '';
   while length(st)>=64
      nn = max(find(st(1:64)==' '));
      if isempty(nn)|nn<50  nn = 64; end
      str = sprintf('%s%s\n',str,st(1:nn));
      st  = st(nn+1:end);
   end
   str = [str,st];
end

% Ausgabe
% =======
sc = find(sclk>0);

if vis(5) soundsc(y,fa); end

if vis(10)
   fh1= fullscreen('Blocksync');
   subplot(3,1,1); plot(t,y);         set(gca,'XLim',[0,t(length(t))])
   subplot(3,1,2); plot(t,real(z));   set(gca,'XLim',[0,t(length(t))])
   subplot(3,1,3); plot(t,real(zsq)); set(gca,'XLim',[0,t(length(t))])
end

if vis(1)
   komplexanimat(z,[1 0 0 0 0 1],10,[90 0],sc)
   title('Signal nach dem Empfangsfilter','FontSize',13); 
   set(gcf,'Name','z')
end

if vis(2)
   komplexanimat(zsq,[1 0 0 0 0 1],10,[90 0],sc)
   title('z^m','FontSize',13); 
   set(gcf,'Name','z^m')
end

if vis(3)
   komplexanimat(zz,[1 0 0 0 0 1],10,[90 0],sc)
   title('Signal nach der Trgerphasenkorrektur','FontSize',13); 
   set(gcf,'Name','Korrektur')
end

if vis(4)
   fh4 = fullscreen('Phase');
   subplot(2,1,1); plot(t(k),unwrap(angle(tsq(k))))
   line(t(k),polyval(c,t(k)),'Color','r')
   set(gca,'XLim',[2*g length(zsq)]/fa)
   title('Trgerphase','FontSize',13)
   subplot(2,1,2); plot(t(k),unwrap(angle(ztsq)))
   line(t(k),polyval(zc,t(k)),'Color','r')
   set(gca,'XLim',[g+100 length(zzsq)]/fa)
   title('Schrittphase','FontSize',13)
end

if vis(6)
   komplexanimat(zz,[1 0 0 0 0 1],10,[90 0],sc)
   title('zz','FontSize',13); 
   set(gcf,'Name','zz')
end

if vis(7)
   komplexanimat(zzsq,[1 0 0 0 0 1],10,[90 0],sc)
   title('zz^m','FontSize',13); 
   set(gcf,'Name','zz^m')
end

if vis(8)
   fh8 = fullscreen('Schritt'); 
   plot([real(schr); imag(schr)]')
   title('Schritt','FontSize',13); 
   set(gcf,'Name','Schritt')
end

if vis(9)
   fh9 = fullscreen('Filter');
   subplot(2,1,1); filterplot(b)
   title('Empfangsfilter','FontSize',13)
   subplot(2,1,2); filterplot({{bs;as}},0,0.1)
   title('Schrittsync-Filter','FontSize',13)
end  