function y = p3dtx(x,r,datei)
% y = p3dtx(x,r,datei)
% Telemetrie-Simulation des Satelliten P3-D (AMSAT OSCAR-40)
%  x :    zu sendende Bits (maximal 4096).
%         Anstelle eines Bitvektors ist auch ein String von 
%         maximal 512 Zeichen erlaubt.
%         Dem Block wird zur Synchronisation folgende Sequenz
%         von 4 Bytes vorangestellt (in HEX): $39 $15 $ED $30
%         Falls mehr als 256 Bits bzw mehr als 32 Zeichen 
%         angegeben sind wird der Block mit Leerzeichen auf
%         512 Bytes aufgefllt und ein CRC von 2 Bytes angefgt.
%  r :    Standardabweichung des weissen Rauschens
%  datei: Dateiname, falls dieser angegeben ist, wird das Signal 
%         in dieser Datei im *.WAV-Format gespeichert
%  y :    PSK-Signal mit Trgerfrequenz 1.5 kHz
% Wenn y einer Variablen zugewiesen wird, gibt es keine Graphik
% und keine Sound-Ausgabe.

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Blockaufbau
% ===========
if nargin<1    x = 'Dies ist der Default-String'; end
if ischar(x)
   z  = double(x);
   xx = [];
   for k = 1:length(z)  xx = [xx,num2binvec(x(k),8)]; end
else 
   xx = x;   
end
if length(xx)>4096 
   warning('Block abgeschnitten!')
   xx = xx(1:4096);
end
if length(xx)>256 
   blk = nfach(double(' '),512);
   blk(1:length(xx)) = xx;
   xx = blk;
   xx = [xx,crc(xx,[16 12 5 0])];              % CRC-Berechnung
   xx = [xx, 0 1 0 1 0 0 0 0 0 1 0 1 0 0 0 0]; % Inter-Block
end

xx = [1 0 1 0 1 1 0 0 0 1 1 1 0 0 1 0 0 0 1 0 1 0 1 1 1 1 0 1 1 0 1 0 0 1 1 0 0 0 0 xx];

% Konstanten
% ==========
fa = 8000;            % Abtastrate 8 kHz
f0 = 1500;            % Trgerfrequenz
fs = 800;             % Schrittfrequenz
b  = fir1(12,fs/fa);  % Entwurf des Sendefilters

% Sender
% ======
z  = filter(b,1,2*nfach(manchester(diffencode(xx)),fa/fs)-1);
y  = z.*cos(2*pi*f0*(0:length(z)-1)/fa);   % DSB-Modulation

% Kanalrauschen
% =============
if nargin>1  y = y + r*randn(size(y)); end % weisses Kanalrauschen

% Ausgabe
% =======
y  = y/max(abs(y));

if nargin>2  wavwrite(y,fa,16,datei); end

if nargout>0  break; end

lx = length(xx);
fh = fullscreen('Basisband');
subplot(5,1,1); plot(nfach(xx,2*fa/fs)); axis([1 2*lx*fa/fs -0.2 +1.2])
subplot(5,1,2); plot(nfach(diffencode(xx),2*fa/fs)); axis([1 2*lx*fa/fs -0.2 +1.2])
subplot(5,1,3); plot(nfach(manchester(diffencode(xx)),fa/fs)); axis([1 2*lx*fa/fs -0.2 +1.2])
subplot(5,1,4); plot(z); axis([1 length(z) -1.2 +1.2])
subplot(5,1,5); plot(y); axis([1 length(y) -1.2 +1.2])

soundsc(y,fa)
