function x0 = simhardsoft(p,k,EbN0dB)
% x0 = simhardsoft(p,k,EbN0dB)
% Demo fr die Bitfehlerstatistik bei der Dekodierung von 
% zyklischen Kodes unter Gauss-Rauschen.
%
% Der Kode wird durch das Polynom p festgelegt. Die Lnge von p 
% bestimmt die Lnge n des Kodes. U.U. sind Nullen anzufgen. 
% Als Minimalabstand des Kodes wird das Gewicht von p genommen.
% Daher ist fr p ein Vektor mit minimalem Gewicht zu whlen.
% Die harte Entscheidung wird nur dann als richtig gewertet, wenn
% wenigstens  n - floor((d-1)/2) Bits richtig entschieden sind.
%
% k ist die Zahl der Informationsbits im Kodewort.
%
% EbN0dB ist Eb/no in dB.
%
% Es werden Histogramme der Fehlerhufigkeit gezeichnet fr 
% folgende Flle:
% (a)  mit harter Bitentscheidung und anschliessender Dekodierung
% (b)  mit Soft-Dekodierung
%
% x0 = [Anzahl Stichproben, davon Dekodierfehler hard, Dekodierfehler soft]
% Wird die Ausgabe auf eine Variable zugewiesen, so unterbleibt die
% Darstellung.
%
% Vorgehen:  Angenommen wird o.B.d.A. die Aussendung des Nullvektors.
% Seine direkt benachbarten n Kodewrter sind gegeben durch die 
% zyklische Verschiebung von p. Es wird geprft, ob der euklidische 
% Abstand des erzeugten Rauschvektors zu diesen Nachbarn durchweg 
% grsser als das Gewicht des Rauschvektors ist. Im Falle von nicht-
% zyklischen Kodes sind die gewhlten Nachbarn des Nullvektors nicht 
% wirkliche Kodewrter. Die Statistik wird aber korrekt dargestellt.

% Defaults, Parameterprfung  und Konstanten
% ==========================================
if nargin<3  EbN0dB = 4 ; end
if nargin<2  k = 12; end
if nargin<1  p = [0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 1 1 1 0 1 0 1]; end
n = length(p);                          % Kodewortlnge
d = sum(p);                             % Minimalabstand
if n<2 error('simhardsoft: Codewortlnge zu klein !'); end
if d<2 error('simhardsoft: Minimalabstand zu klein !'); end
if k<0 error('simhardsoft: k < 0 ist unsinnig.'); end
fsiz = 14;                              % FontSize
m = floor(16384/n);                     % Anzahl der Stichproben
sigma = 1/sqrt(2*k/n*(10^(EbN0dB/10))); % Standardabweichung
a = 0.5;

% Kodewortnachbarn
% ================
cw = zyktoeplitz(2*p)';                 % zyklische Toeplitzmatrix {+2}

% Rauschen
% ========
noise = randn(n,m)*sigma;               % m Rauschvektoren der Lnge n
eugew = sum(noise.^2);                  % deren Gewicht (Energie) 
   
% Hard Decision   ( hd > Minimalabstand/2 bedeutet Dekodierfehler )
% =============
hd = sum(noise>1);                      % nur Fehler in einer Richtung zhlen
shd= sum(hd>floor((d-1)/2));            % Anzahl Dekodierfehler

% Soft Decision   ( sd > 0  bedeutet Dekodierfehler )
% =============
for k=1:m  
   sd(k) = (eugew(k) - min(sum(((ones(n,1)*noise(:,k)' - cw).^2)')))/n;
end
ssd = sum(sd>0);                        % Anzahl Dekodierfehler

% Ausgabe
% =======
x0 = [m shd ssd];
if nargout>0 break; end                 % falls Ausgabe dann fertig

% Darstellung Hard-Decision
% =========================
ug = d/2;
mfs= max(hd);
[hh,xh] = hist(hd,(0:mfs+1));
ig = find(xh<ug);
ir = find(xh>=ug);
xg = xh(ig);
hg = hh(ig);
xr = xh(ir);
hr = hh(ir);
zg = zeros(1,length(hg));
zr = zeros(1,length(hr));

fh = fullscreen('Rausch-Histogramme');
subplot(2,1,1)
patch([xg-a;xg+a;xg+a;xg-a],[zg;zg;hg;hg],'g')
patch([xr-a;xr+a;xr+a;xr-a],[zr;zr;hr;hr],'r')
line(ug,0,'Marker','o','MarkerSize',12,'MarkerFaceColor','c')
set(gca,'XLim',[-a,mfs+2],'FontSize',fsiz)
title(sprintf(...
'Hard Decoding:  Anteil Dekodierfehler = %1.4f  (Stichprobe von %4d)',...
   shd/m,m),'FontSize',fsiz)
xlabel('Anzahl Bitfehler im Kodewort','FontSize',fsiz)
ylabel('Vorkommnisse','FontSize',fsiz)

% Darstellung Soft-Decision
% =========================
von = -2.625;
srt = 0.25;
bis = +2.625;
ug  = 11;
a   = 0.125;
if max(sd)>2.5
   fak = 2^ceil(log2(max(sd)));
   von = von*fak;
   srt = srt*fak;
   bis = bis*fak;
   a   = a*fak;
end
[hs,xs] = hist(sd,von:srt:bis);      % Soft-Decision
xs = [xs,zeros(1,ug-length(xs))];
hs = [hs,zeros(1,ug-length(hs))];
lr = ug + 1;
lx = length(xs);
xg = xs(1:ug);
hg = hs(1:ug);
xr = xs(lr:lx);
hr = hs(lr:lx);
zg = zeros(1,length(hg));
zr = zeros(1,length(hr));

subplot(2,1,2)
patch([xg-a;xg+a;xg+a;xg-a],[zg;zg;hg;hg],'g')
patch([xr-a;xr+a;xr+a;xr-a],[zr;zr;hr;hr],'r')
line(0,0,'Marker','o','MarkerSize',12,'MarkerFaceColor','c')
line(-2,0,'Marker','o','MarkerSize',12,'MarkerFaceColor','g')
set(gca,'XLim',[min(xs),max(xs)],'FontSize',fsiz)
title(sprintf(... 
'Soft Decoding:  Anteil Dekodierfehler = %1.4f  (Stichprobe von %4d)',...
   ssd/m,m),'FontSize',fsiz)
xlabel('Energieabstand von der Entscheidungsschwelle','FontSize',fsiz)
ylabel('Vorkommnisse','FontSize',fsiz)
