function d = viterbiq(q,g,x,diag)
% d = viterbiq(q,g,x,s,diag)
% Viterbi-Dekodierung von Faltungskodes ber dem GF(q).
% Aufruf ohne Parameter liefert eine interaktive Graphik-Demo.
%
%   q  :   Anzahl der verwendeten Symbole
%
%   g  :   Generatormatrix mit r Faltungspolynomen als Zeilen {0...q-1}.
%          Anstelle der Generatormatrix kann  g = [cr, cl] eingegeben werden,
%          wo cr die Anzahl der Polynome und cl die Constraint Length 
%          bedeuten. Dann wird eine zuflige Generatormatrix erzeugt.
%
%   x  :   Matrix des zu dekodierenden Signals: Fr jede Abtastung liegt
%          ein Vektor x(:,1:q) bzw. x(:,:,1:q) vor mit den empangenen 
%          Energien fr die einzelnen Symbole 0...q-1.
%          Ist x zweidimensional, so wird angenommen, dass die Faltung 
%          mit allen Polynomen verzahnt vorliegt.
%          x kann auch unverzahnt als  size(g)*q-Matrix  angegeben werden.
%
%  diag:   falls angegeben mit beliebigem Wert~=0: Trellisgraphik 
%          und Kontrollausgabe ins Kommandofenster.
%          Dies ist nur sinnvoll fr Constraint Length < 5 !
%          Diese Ausgaben gibt es auch, falls viterbi ganz ohne
%          Parameter aufgerufen wird.
%
%   d  :   dekodierte Symbole {0...q}
%
% Hinweis: Falls mit dem Trellis-Diagramm gearbeitet wird, erfolgt 
%          die Berechnung und Ausgabe der ersten zwei Schritte durch  
%          Fortschaltung mit beliebiger Tatatureingabe. Nach der 
%          vollstndigen Dekodierung wird der verbliebene Weg im 
%          Trellisdiagramm rot eingezeichnet. Dann kann das Diagramm
%          mit den Tasten -+ seitlich verschoben werden.
%
%          Die Berechnung des Trellisdiagramms ist ausserordentlich 
%          rechenintensiv !
%
%          Smtliche Trelliswege werden nur bis zu einer Lnge von 
%          144 Bit verwaltet. ber diese Lnge hinaus darf nur noch 
%          ein endgltig entschiedener Weg existieren. Andernfalls 
%          wird eine Warnung ausgegeben, die immer ein Zeichen fr 
%          ein zu stark gestrtes Signal ist oder eines, das nicht 
%          zum angegebenen Faltungspolynom passt.
%

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<4 diag = 0; end
if nargin<2
   diag = 1;
   g = [2 2];
end
if nargin<1    q = 3; end
[cr,cl] = size(g);
if cr==1
   cr = g(1);
   cl = g(2);
   clear g;
   switch q
      case 2,  g = faltpoly(2,3);
      case 3,  
         for k=[1,cl]  g(:,k) = ceil((q-1)*rand(cr,1)); end % das ist vorerst nur geraten
         for k=2:cl-1  g(:,k) = floor(q*rand(cr,1));    end
      otherwise,  
         for k=[1,cl]  g(:,k) = ceil((q-1)*rand(cr,1)); end % das ist vorerst nur geraten
         for k=2:cl-1  g(:,k) = floor(q*rand(cr,1));    end
   end
   Generatormatrix = g   
end
if nargin<3
   ebn0 = 2.0;
%   info = floor(q*rand(1,8))
   info = floor(q*rand(1,14));
   code = faltcode(info,g,0,q);
   [m,n]= size(code);
   ids  = zeros(m,n,q);
   snr  = ebn0*log2(q)*(n-cl+1)/n/cr;  % SNR fr einzelnes Symbol (linear)
   x    = randn(m,n,q).^2;             % Rauschenergie  1.0/Kanal/Schritt
   for km=1:m
      for kn=1:n
         ids(km,kn,1+code(km,kn)) = ids(km,kn,1+code(km,kn)) + 1;
           x(km,kn,1+code(km,kn)) =   x(km,kn,1+code(km,kn)) + snr; 
      end
   end
end
if nargin<2 | diag
   Generatormatrix  = qmat2str(g,q)
   if nargin<1  Informationssymbole = qmat2str(info,q), end
   Kodesignale_Stichprobe = x(:,min(8,end));
   if q^cl>32 | length(x)>10 
      Hinweis = 'Trellis-Diagramm nicht sinnvoll !'
      diag = 0;
   end
end

% Konstanten
% ==========
stellen = floor(17*log(2)/log(q)); % Stellen zur Basis q in Wegekompression
s = 0:q-1;                       % Sollwerte
[cr,cl] = size(g);               % inverse Koderate cr und Constraint Length cl
[xnz,xns,qq] = size(x);
if xnz==1  
   if rem(xns,cr)  error('Lnge von x passt nicht zu g'); end
   x = reshape(x,cr,xns/cr);     % x zu Matrix mit cr Zeilen machen
elseif xnz~=cr  
   error('Anzahl der Zeilen von x ungleich der von g');   
end
x = [x ones(cr,1,q)*s(1)];       % eine Null-Spalte an Tail anhngen
[xnz,xns,qq] = size(x);
fsz  = 13;                       % FontSize fr Darstellung
qhcl = q^cl;                     % Anzahl der Zustnde
qhcl1= q^(cl-1);                 % 
for k=1:qhcl                     % das letzte Symbol der Kodierung aller Zustandsmuster
   y = faltcode(reverse(num2basvec(k-1,q,cl)),g,0,q);
   for r=1:cr  z(r,k) = y(r,cl) + 1; end;
end
kq = reshape(1:q*qhcl1,qhcl1,q);
pq = ones(q,1)*(0:qhcl1-1)*q + (1:q)'*ones(1,qhcl1);

% Trellisanfang
% =============
for k=1:qhcl                     % alle Zustandsmuster als Anfangswege (reverse)
   uu    = reverse(num2basvec(k-1,q,cl));
   bw(k) = basvec2num(uu,q);
   xi    = 1 + faltcode(uu,g,0,q);
   sx    = 0;
   for km=1:xnz  
      for kn=1:cl 
         sx = sx + x(km,kn,xi(km,kn)); % Energiesumme 
      end; 
   end
   c(k)  = sx;
end
zw1 = zeros(1,qhcl);             % komprimierte Wegereprsentation
zw2 = zeros(1,qhcl);             % komprimierte Wegereprsentation
zw3 = zeros(1,qhcl);             % komprimierte Wegereprsentation
zlr = cl;                        % Zhler fr Wegekompression
zl2 = 0;                         % Zhler fr Wegekompression
d   = [];
rvk1= ones(1,qhcl);              % Rckverkettung
rvk2= ones(1,qhcl);              % Rckverkettung
rvk3= ones(1,qhcl);              % Rckverkettung
% Kontrollausgabe und Trellis-Graphik
if diag 
   fh = trellisdiag(cl,q);
   for k=1:qhcl
      addtrellisdiag(fh,cl,q,1,0,num2basvec(bw(k),q,cl),0.8*[1 1 1]);
   end      
end

% Trellisschritt
% ==============
for t=cl+1:xns                   % Zeitschleife ber Kodelnge
   for k=1:q  cq(k,:) = c(pq(k,:)); end; % Energie der Vorgngerzustnde
   [mc,mi] = max(cq);            % Entscheidung fr maximale Energie
   for km=1:qhcl                 % alle Zustnde zur Zeit t
      kmm  = rem(km-1,qhcl1) + 1;% Gruppen von qhcl1 mit den selben Vorgngern
      kdd  = floor((km-1)/qhcl1) + 1; % Gruppen von qhcl1 mit den selben letzten Symbolen
      pqi  = pq(mi(kmm),kmm);    % gewhlte Wege
      bwn(km) = q*bw(pqi) + kdd - 1;  % neue Wegekompression
      sq = 0;
      for r=1:cr 
         sq = sq + x(r,t,z(r,km));    % Summe der Energien der Strnge
      end
      c(km)    = mc(kmm) + sq;   % neue Energiesumme
      rvkn(km) = rvk1(pqi);      % neue Kompressions-Rckverkettung
   end
   bw   = bwn;
   rvk1 = rvkn;
   % Wegekompression
   zlr = zlr + 1;
   if rem(zlr,stellen)==0 
      if zlr==3*stellen
         rvk = rvk3(rvk2(rvk1(1)));
         if any(rvk~=rvk3(rvk2(rvk1(2:qhcl)))) 
            warning(sprintf('Weg nach %3d Schritten noch nicht entschieden.',3*(stellen-1)-1))
         end
         d = [d  num2basvec(zw3(rvk),q,3*stellen)];
         zw3 = zw2;
         zw2 = zw1;
         zw1 = bw';
         rvk3= rvk2;
         rvk2= rvk1;
         rvk1= 1:qhcl;
         zlr = 0;
      end    
      bw  = rem(bw,q^(2*stellen));        
   end
   % Graphik
   if diag                       % Trellisdiagramm
      mxc = max(c);
      mnc = min(c);
      if t<=length(x)+1
         for k=1:qhcl
            addtrellisdiag(fh,cl,q,1,0,num2basvec(bw(k),q,t),...
                           (mxc-c(k))/(mxc-mnc)*[1 1 1]);
         end
      end
      if t<cl+4  pause; end   
   end
end

% Ausgabe
% =======
d = [d  num2basvec(zw3(rvk3(rvk2(rvk1(1)))),q,3*stellen)   ...
        num2basvec(zw2(rvk2(rvk1(1))),q,3*stellen)  ...
        num2basvec(zw1(rvk1(1)),q,3*stellen)  ...
        num2basvec(rem(bw(1),q^zlr),q,zlr)];
d = d(9*stellen+1:end-cl);
if diag                         % Trellisdiagramm
   addtrellisdiag(fh,cl,q,1,0,d,'r');
end

if nargout>0  
   if nargin<3
      d = sum(d~=info);    
   end
   break; 
end

% Kontrollausgabe
% ===============
Energiesumme = c(1)
Energiereal  = sum(sum(sum(ids.*x(:,1:end-1,:))))
[mx,nx,ox] = size(x);
nx = nx - 1;
cc = faltcode(d,g,0,q);
cx = zeros(mx,nx,ox);
for k=1:cr  for t=1:nx  cx(k,t,1+cc(k,t)) = 1;  end; end
%Checksum = sum(sum(sum(cx.*x(:,1:end-1,:))))
Rauschenprosymbol = sum(sum(sum((1-cx).*x(:,1:end-1,:))))/cr/nx/(q-1)
Energieprosymbol  = c(1)/cr/nx - Rauschenprosymbol
Symbolfehler      = sum(d~=info)

mx = max(max(max(x)));
for kr=1:cr
   fhd(cr) = fullscreen('Signale','w',2);
   for k=1:q
      axes('Position',[0 (k-1)/q 1 1/q]);
      plot(1:nx+1,x(kr,:,k),1:nx,0.6*mx*cx(kr,:,k),'r*',1:nx,0.6*mx*ids(kr,:,k),'mo')
      set(gca,'YLim',[0 1.1*mx])
   end
end