function nf = viterbitest(g,sigma,n)
% nf = viterbitest(g,sigma,n)
% Demo fr Viterbi-Dekoder
%    g  :  Generatorpolynome fr Faltungskode
% sigma :  Standardabweichung des weissen Rauschens
%    n  :  Anzahl der Informationsbits
%
% Es werden n zufllige Informationsbits mit den durch g 
% gegebenen Polynomen faltungskodiert. Nach Addition des 
% Rauschens wird das Signal dekodiert und die Anzahl der 
% Bitfehler in der Information ausgegeben.
% Ausgegeben wird auch die zur Dekodierung bentigte CPU-Zeit.
%
% Als Graphik wird die Signalverteilung dargestellt mit den 
% beiden Normalverteilungen um +1 und -1 (mit Sigma) und mit 
% deren Summe.
% Wenn nf einer Variablen zugewiesen wird, gibt es keine Graphik.

% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de 
%
%
% Defaults
% ========
if nargin<3      n = 1000; end
if nargin<2  sigma = 0.8;  end
if nargin<1      g = faltpoly(2,9); end

% Konstanten
% ==========
fsz = 13;                       % FontSize
del = ceil(5*sigma)/10;         % Auflsung der Fehlerverteilung
sgr = 1 + 7*del;                % Darstellungsbereich der Verteilung
[r,cl] = size(g);               % Parameter der Faltungspolynome

% Kodierung und Kanalsimulation
% =============================
u = randb(n);                   % zufllige Information
x = 2*faltcode(u,g)-1;          % Faltungskodierung
y = x + sigma*randn(size(x));   % Kanalrauschen
xx  = reshape(x,1,r*length(x)); % Verzahnung fr Signaldarstellung
yy  = reshape(y,1,r*length(y)); % Verzahnung fr Signaldarstellung

% Dekodierung
% ===========
t0  = clock;
d   = viterbi(g,yy,[-1 +1]);    % Dekodierung
nf  = sum(d~=u);                % Anzahl der Bitfehler in der dekodierten Information

if nargout>0  break; end

cpuzeit = etime(clock,t0)       % Ausgabe der CPU-Zeit ins Kommandofenster

% Signaldarstellung
% =================
if n<257
   fh1 = fullscreen('Signal');
   plot([xx;yy]'); axis([0 length(xx) -sgr +sgr])
   line([0,length(xx)],[0 0],'LineStyle',':','Color',0.7*[1 1 1])
end

% Darstellung der Fehlerverteilung
% ================================
fh2 = fullscreen('Verteilung');
set(gca,'Position',[0.05 0.1 0.915 0.85])
hist(yy,-sgr:del:+sgr)          % Darstellung der Fehlerverteilung
n1  = sum(flach(x)==+1);        % Anzahl der +1
n0  = sum(flach(x)==-1);        % Anzahl der -1
s   = -sgr:del/20:+sgr;         % Darstellungsbereich
%yd  = 0.0008*(n+r*cl);          % vertikale Verschiebung einer Linie
yd  = 0.0008*r*(n+cl);          % vertikale Verschiebung einer Linie
g1  = exp(-((s-1).^2)/(2*sigma^2))*n1*del/(sqrt(2*pi)*sigma); % Gauss-Verteilung
g0  = exp(-((s+1).^2)/(2*sigma^2))*n0*del/(sqrt(2*pi)*sigma); % Gauss-Verteilung
line(s,g0+g1+yd,'Color','w')    % Sichtbarmachen im Histogramm
line(s,g0+g1,'Color','k')       % Summenverteilung
line(s,g1,'Color','r')          % Gauss-Verteilung der +1-Signale
line(s,g0,'Color','g')          % Gauss-Verteilung der -1-Signale
%set(gca,'XLim',[-6.25 +6.25])
xlabel('Verteilungen des Signalwertes;  rot: binre 1;  grn: binre 0;  schwarz/weiss: Summe;  blau: aktuelle Stichprobe','FontSize',13) 
ax = axis;
text(0.88*ax(1),0.91*ax(4),'Faltungskode:','FontSize',fsz+5)
text(0.88*ax(1),0.85*ax(4),sprintf('Constraint Length:  %2d',cl),'FontSize',fsz)
text(0.88*ax(1),0.80*ax(4),sprintf('Kode-Rate:            1/%d',r),'FontSize',fsz)
text(0.88*ax(1),0.75*ax(4),sprintf('Sigma:                 %1.3f',sigma),'FontSize',fsz)
text(0.88*ax(1),0.70*ax(4),sprintf('Informationsbits:   %4d',n),'FontSize',fsz)
text(0.88*ax(1),0.65*ax(4),sprintf('Bitfehlerrate:    %1.5f',nf/n),'FontSize',fsz)
